unit CiaConsole;

interface

uses
  Messages, SysUtils, Classes, Controls, StdCtrls;

type
  TCiaConsole = class(TMemo)
  private
    FEcho: boolean;
    FKeyPress: TKeyPressEvent;
    FMaxLines: integer;
    FMinLines: integer;
  protected
    procedure KeyPress(var Key: char); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure   Add(const _Rcvd: string);
  published
    property Echo: boolean read FEcho write FEcho;
    property MaxLines: integer read FMaxLines write FMaxLines;
    property MinLines: integer read FMinLines write FMinLines;
    property OnKeyPress: TKeyPressEvent read FKeyPress write FKeyPress;
  end;

procedure Register;

implementation

constructor TCiaConsole.Create(AOwner: TComponent);
begin
   inherited;
   FEcho := True;
   FMaxLines := 400; // 400 lines of 80 characters = 32 k
   FMinLines := 300;
   Font.Name := 'Courier New';
   Font.Size := 8;
end;

procedure TCiaConsole.KeyPress(var Key: char);
begin
   if assigned(FKeyPress) then
      FKeyPress(self, Key);

   if FEcho then
      Add(Key);
   Key := #0;
   inherited;
end;

procedure TCiaConsole.Add(const _Rcvd: string);
var
   i: integer;
   Rcvd: string;
begin
   if Lines.Count = 0 then
      Lines.Add('');

   if (FMaxLines > 0) and (Lines.Count > FMaxLines) then
   begin
      SelStart := 0;
      SelLength := Perform(EM_LINEINDEX, Lines.Count - FMinLines, 0);
      SelText := EmptyStr;
      SelStart := GetTextLen;
   end;


   for i := 1 to length(_Rcvd) do   // CrLf has to be send together to the memo
      if _Rcvd[i] <> #10 then
      begin
         Rcvd := Rcvd + _Rcvd[i];
         if _Rcvd[i] = #13 then
            Rcvd := Rcvd + #10;
      end;

   while (length(Rcvd) > 0) and (Rcvd[1] = #8) do
   begin
      if length(Lines[Lines.Count - 1]) > 0 then
         Lines[Lines.Count - 1] := Copy(Lines[Lines.Count - 1], 1, length(Lines[Lines.Count - 1]) - 1);
      Delete(Rcvd, 1, 1);
   end;

   i := Pos(#8, Rcvd);
   while i > 0 do
   begin
      if (i > 1) and (Rcvd[i - 1] <> #10) then
         Delete(Rcvd, i - 1, 2)
      else
         Delete(Rcvd, i, 1);
      i := Pos(#8, Rcvd);
   end;

   Lines[Lines.Count - 1] := Lines[Lines.Count - 1] + Rcvd;
end;

procedure Register;
begin
  RegisterComponents('Cia', [TCiaConsole]);
end;

end.
